@@ -4,40 +4,37 @@
 lda <- function(x, ...) UseMethod("lda")
 
 
-lda.formula <- function(formula, data = NULL, ...,
-			subset, na.action = na.fail)
+lda.formula <- function(formula, data, ..., subset, na.action)
 {
     m <- match.call(expand.dots = FALSE)
-    if(is.matrix(eval.parent(m$data)))
-        m$data <- as.data.frame(data)
+#    if(is.matrix(eval.parent(m$data))) # done in model.frame.default
+#        m$data <- as.data.frame(data)
     m$... <- NULL
     m[[1]] <- as.name("model.frame")
     m <- eval.parent(m)
     Terms <- attr(m, "terms")
-    grouping <- model.extract(m, "response")
+    grouping <- model.response(m)
     x <- model.matrix(Terms, m)
-    xvars <- as.character(attr(Terms, "variables"))[-1]
-   if ((yvar <- attr(Terms, "response")) > 0) xvars <- xvars[-yvar]
-    xlev <- if (length(xvars) > 0) {
-        xlev <- lapply(m[xvars], levels)
-        xlev[!sapply(xlev, is.null)]
-    }
     xint <- match("(Intercept)", colnames(x), nomatch=0)
     if(xint > 0) x <- x[, -xint, drop=FALSE]
     res <- lda.default(x, grouping, ...)
     res$terms <- Terms
-    res$call <- match.call()
+    ## fix up call to refer to the generic, but leave arg name as `formula'
+    cl <- match.call()
+    cl[[1]] <- as.name("lda")
+    res$call <- cl
     res$contrasts <- attr(x, "contrasts")
-    res$xlevels <- xlev
-    attr(res, "na.message") <- attr(m, "na.message")
-    if(!is.null(attr(m, "na.action"))) res$na.action <- attr(m, "na.action")
+    res$xlevels <- .getXlevels(Terms, m)
+    res$na.action <- attr(m, "na.action")
     res
 }
 
 lda.data.frame <- function(x, ...)
 {
-    res <- lda.matrix(structure(data.matrix(x), class="matrix"), ...)
-    res$call <- match.call()
+    res <- lda(structure(data.matrix(x), class="matrix"), ...)
+    cl <- match.call()
+    cl[[1]] <- as.name("lda")
+    res$call <- cl
     res
 }
 
@@ -55,8 +52,11 @@ lda.matrix <- function(x, grouping, ...,
         grouping <- dfr$g
         x <- dfr$x
     }
-    res <- NextMethod("lda")
-    res$call <- match.call()
+#    res <- NextMethod("lda")
+    res <- lda.default(x, grouping, ...)
+    cl <- match.call()
+    cl[[1]] <- as.name("lda")
+    res$call <- cl
     res
 }
 
@@ -139,7 +139,7 @@ lda.default <-
         K <- if(method == "moment") ng else 0
         dist <- matrix(0, n, ng)
         for(i in 1:ng) {
-            dev <- x - matrix(dm[i,  ], n, p, byrow = TRUE)
+            dev <- x - matrix(dm[i,  ], n, rank, byrow = TRUE)
             dist[, i] <- rowSums(dev^2)
         }
         ind <- cbind(1:n, g)
@@ -147,7 +147,7 @@ lda.default <-
         cc <- nc/((nc-1)*(n-K))
         dist2 <- dist
         for(i in 1:ng) {
-            dev <- x - matrix(dm[i,  ], n, p, byrow = TRUE)
+            dev <- x - matrix(dm[i,  ], n, rank, byrow = TRUE)
             dev2 <- x - dm[g, ]
             tmp <- rowSums(dev*dev2)
             dist[, i] <- (n-1-K)/(n-K) * (dist2[, i] +  cc*tmp^2/(1 - cc*dist2[ind]))
@@ -174,9 +174,11 @@ lda.default <-
         dimnames(scaling) <- list(colnames(x), paste("LD", 1:rank, sep = ""))
         dimnames(group.means)[[2]] <- colnames(x)
     }
+    cl <- match.call()
+    cl[[1]] <- as.name("lda")
     structure(list(prior = prior, counts = counts, means = group.means,
                    scaling = scaling, lev = lev, svd = X.s$d[1:rank],
-                   N = n, call = match.call()),
+                   N = n, call = cl),
               class = "lda")
 }
 
@@ -186,14 +188,16 @@ predict.lda <- function(object, newdata, prior = object$prior, dimen,
     if(!inherits(object, "lda")) stop("object not of class lda")
     if(!is.null(Terms <- object$terms)) { #
     # formula fit
+        Terms <- delete.response(Terms)
         if(missing(newdata)) newdata <- model.frame(object)
         else {
-            newdata <- model.frame(as.formula(delete.response(Terms)),
-                                   newdata, na.action=function(x) x,
+            newdata <- model.frame(Terms, newdata, na.action=na.pass,
                                    xlev = object$xlevels)
+            if (!is.null(cl <- attr(Terms, "dataClasses")) &&
+                exists(".checkMFClasses", envir=NULL))
+                .checkMFClasses(cl, newdata)
         }
-        x <- model.matrix(delete.response(Terms), newdata,
-                          contrasts = object$contrasts)
+        x <- model.matrix(Terms, newdata, contrasts = object$contrasts)
         xint <- match("(Intercept)", colnames(x), nomatch=0)
         if(xint > 0) x <- x[, -xint, drop=FALSE]
     } else { #
@@ -201,8 +205,9 @@ predict.lda <- function(object, newdata, prior = object$prior, dimen,
         if(missing(newdata)) {
             if(!is.null(sub <- object$call$subset))
                 newdata <-
-                    eval.parent(parse(text=paste(deparse(object$call$x),"[",
-                                      deparse(sub),",]")))
+                    eval.parent(parse(text=paste(deparse(object$call$x,
+                                      backtick=TRUE),
+                                      "[", deparse(sub, backtick=TRUE),",]")))
             else newdata <- eval.parent(object$call$x)
             if(!is.null(nas <- object$call$na.action))
                 newdata <- eval(call(nas, newdata))
@@ -292,7 +297,7 @@ plot.lda <- function(x, panel = panel.lda, ..., cex=0.7,
     # formula fit
         data <- model.frame(x)
         X <- model.matrix(delete.response(Terms), data)
-        g <- model.extract(data, "response")
+        g <- model.response(data)
         xint <- match("(Intercept)", colnames(X), nomatch=0)
         if(xint > 0) X <- X[, -xint, drop=FALSE]
     } else { #
@@ -300,10 +305,10 @@ plot.lda <- function(x, panel = panel.lda, ..., cex=0.7,
         xname <- x$call$x
         gname <- x$call[[3]]
         if(!is.null(sub <- x$call$subset)) {
-            X <- eval.parent(parse(text=paste(deparse(xname),"[",
-                                   deparse(sub),",]")))
-            g <- eval.parent(parse(text=paste(deparse(gname),"[",
-                                   deparse(sub),"]")))
+            X <- eval.parent(parse(text=paste(deparse(xname, backtick=TRUE),
+                                   "[", deparse(sub, backtick=TRUE),",]")))
+            g <- eval.parent(parse(text=paste(deparse(gname, backtick=TRUE),
+                                   "[", deparse(sub, backtick=TRUE),"]")))
         } else {
             X <- eval.parent(xname)
             g <- eval.parent(gname)
@@ -398,15 +403,11 @@ pairs.lda <- function(x, labels = colnames(x), panel = panel.lda,
         text(x, y, as.character(g.lda), cex=tcex, ...)
     }
     type <- match.arg(type)
-    if(type == "trellis") {
-        warn("type = trellis is not supported in R")
-        type <- "std"
-    }
     if(!is.null(Terms <- x$terms)) { #
     # formula fit
         data <- model.frame(x)
         X <- model.matrix(delete.response(Terms), data)
-        g <- model.extract(data, "response")
+        g <- model.response(data)
         xint <- match("(Intercept)", colnames(X), nomatch=0)
         if(xint > 0) X <- X[, -xint, drop=FALSE]
     } else { #
@@ -414,13 +415,13 @@ pairs.lda <- function(x, labels = colnames(x), panel = panel.lda,
         xname <- x$call$x
         gname <- x$call[[3]]
         if(!is.null(sub <- x$call$subset)) {
-            X <- eval.parent(parse(text=paste(deparse(xname),"[",
-                                   deparse(sub),",]")))
-            g <- eval.parent(parse(text=paste(deparse(gname),"[",
-                                   deparse(sub),"]")))
+            X <- eval.parent(parse(text=paste(deparse(xname, backtick=TRUE),
+                                   "[", deparse(sub, backtick=TRUE),",]")))
+            g <- eval.parent(parse(text=paste(deparse(gname, backtick=TRUE),
+                                   "[", deparse(sub, backtick=TRUE),"]")))
         } else {
             X <- eval.parent(xname)
-            g <- eval.parent(gnamew)
+            g <- eval.parent(gname)
         }
         if(!is.null(nas <- x$call$na.action)) {
             df <- data.frame(g = g, X = X)
@@ -436,31 +437,31 @@ pairs.lda <- function(x, labels = colnames(x), panel = panel.lda,
     means <- colMeans(x$means)
     X <- scale(X, center=means, scale=FALSE) %*% x$scaling
     if(!missing(dimen) && dimen < ncol(X)) X <- X[, 1:dimen]
-    if(type=="std") pairs.default(X, panel=panel, ...)
+    if(type == "std") pairs.default(X, panel=panel, ...)
     else {
-        print(splom(~X, groups = g, panel=panel.superpose,
-                    key = list(
-                    text=list(levels(g)),
-                    points = Rows(trellis.par.get("superpose.symbol"),
-                    seq(along=levels(g))),
-                    columns = min(5, length(levels(g)))
-                    )
+        print(lattice::splom(~X, groups = g, panel = lattice::panel.superpose,
+                             key = list(
+                             text=list(levels(g)),
+                             points = lattice::Rows(lattice::trellis.par.get("superpose.symbol"),
+                             seq(along=levels(g))),
+                             columns = min(5, length(levels(g)))
+                             )
                     ))
     }
     invisible(NULL)
 }
 
-model.frame.lda <-
-function(formula, data = NULL, na.action = NULL, ...)
+model.frame.lda <- function(formula, ...)
 {
     oc <- formula$call
     oc$prior <- oc$tol <- oc$method <- oc$CV <- oc$nu <- NULL
     oc[[1]] <- as.name("model.frame")
-    if(length(data)) {
-        oc$data <- substitute(data)
-        eval.parent(oc)
+    if(length(dots<- list(...))) {
+        nargs <- dots[match(c("data", "na.action", "subset"), names(dots), 0)]
+        oc[names(nargs)] <- nargs
     }
-    else eval(oc, list())
+    if (is.null(env <- environment(formula$terms))) env <- parent.frame()
+    eval(oc, env)
 }
 
 coef.lda <- function(object, ...) object$scaling
