@@ -50,7 +50,6 @@ trsurf <- trmat(topo.ls, 0, 6.5, 0, 6.5, 30)
 image(trsurf)
 points(topo)
 
-library(modreg)
 par(mfcol = c(2, 2), pty = "s")
 topo.loess <- loess(z ~ x * y, topo, degree = 2, span = 0.25,
   normalize = F)
@@ -115,12 +114,12 @@ points(topo)
 title("GLS trend surface")
 
 prsurf <- prmat(topo.gls, 0, 6.5, 0, 6.5, 50)
-eqscplot(trsurf, , xlab = "", ylab = "", type = "n")
-contour(trsurf, levels = seq(600, 1000, 25), add = T)
+eqscplot(prsurf, , xlab = "", ylab = "", type = "n")
+contour(prsurf, levels = seq(600, 1000, 25), add = T)
 points(topo)
 title("Kriging prediction")
 sesurf <- semat(topo.gls, 0, 6.5, 0, 6.5, 30)
-eqscplot(trsurf, , xlab = "", ylab = "", type = "n")
+eqscplot(sesurf, , xlab = "", ylab = "", type = "n")
 contour(sesurf, levels = c(20, 25), add = T)
 points(topo)
 title("Kriging s.e.")
